/*
	Control View for LControls
	
	This view is double buffered.. fully using Be's Dano BView API
	As such, this will not run on anything not 100% compatible with
	that API. 
	
	BeOS R5 is not compatible, as is not Haiku at this time.
	
	However, one could allow for R5 / Haiku without much extra effort, except 
	without double buffering. Double buffering is vital, turn it off to see why.
	
	This class does not do anything other than handle coms with the controls.
	
	You can override any of the virtual functions (all of them, in fact) I'd suggest
	you do an LView::OverriddenFunction() from your derived classes	however, otherwise
	you WILL lose functionality.
	
	This class is meant to be fast and as light as possible.
	
	WHY USE THIS INSTEAD A BVIEW?
		When you add a BView to another BView:
		
		1) drawing of the underlying view is COMPLETELY obstructed by the possibly empty
			contents of the child.  LView does that, but the controls do not.
		
		2) There is a limit on the number of BViews that can be added to a window/view
			we are only limited by the amount of RAM you wish to use to store the
			4KB of data stored for each control (less than a BView).  Unless BList
			has some limit somewhere :-)
			
		3) Navigation.  As this is meant to control LControls, which are best described
			as instructions to control the contents of the host, and every LControl has
			knowledge of its focus state (true/false) we can handle full navigation.
			You can tab amongst the controls that declare they RequireFocus(). Controls
			that do not need focus are controls such as status bars, which only draw
			information into the host.  Those would	be items that have no need for user
			input from the keyboard.  They will still received mouse notifications
			unless the bool NeedsMouse() is overridden to return false.
			
		4) Flexibility.  The members are stored in a BList, which you have access to
			via the GetMemberList() function.  You can perform sort operations and the
			like.  Just ensure you Lock() first. and Unlock() afterwards.  Every function
			will exit during this lockout if called.  This is to ensure we do not peruse
			the list while your changing it.  You will not be able to get the size even
			from this API, it will ALL fail.  It is a complete lock of operations.
			This can be used to change the order of which controls get checked on 
			first during each operation.  Also can modify navigational order and drawing
			order (say you want one control to come from beneath another to the top.)

	LOCKOUT NOTES:
	
		While all functions exit immediately when locked, DetachedFromWindow()
		and the destructor will spin on the lock awaiting release.  There is
		no timeout, so ensure you unlock as soon as possible in case the user wants
		to close the program.  Spinning also now occurs on AddMember().
		
		When Lock() is called, all ongoing operations will cease nearly instantly.
		This is done with an extra check in all loops when dealing with the members.

		You can prevent the lockouts by using SetFuncLockout(false).
	
	MEMBER STORAGE NOTES:
		
		The BList is constructed to work in increments of one, not the default of 20 (80 KB).
		That means that we will not use unneeded memory, but it also means that we spend 
		more time incrementing the memory space.
		The last argument in the constructor can be used to change this. It is best advised
		to set it to the max # of members you expect to have.  Do not go lower, as you can
		end up using much more RAM than needed, same if you go too much higher.
		
*/

#ifndef L_VIEW_H_
#define L_VIEW_H_

#include <View.h>
class BList;

class LView : public BView{
	friend class LControl;	// this is what WE control, hehe
	public:
		LView(BRect r, 
				const char*, 
				uint32 = B_FOLLOW_ALL, // most commonly used && a must if child of BWindow
				uint32 = B_WILL_DRAW | B_PULSE_NEEDED, 
				int32 ramAllocIncrement = 1);
		virtual ~LView();	// frees list, and all members
		
		virtual void	DetachedFromWindow();
		virtual void	Pulse();

		// Drawing
		virtual void	Draw(BRect);

		// Input Events:
		virtual void	MouseMoved(BPoint, uint32, const BMessage*);
		virtual void	MouseDown(BPoint);
		virtual void	MouseUp(BPoint);
		virtual void	KeyDown(const char*, int32);
		virtual void	KeyUp(const char*, int32);

		// Messaging events		
		virtual void	MessageReceived(BMessage*);
		
		// Locking - see notes above, this locks all operations on the list
		// functions exit, except on destruction, when we spin on the lock
		void	Lock();
		bool	IsLocked();
		void	Unlock();
		void	SetFuncLockout(bool = true);
		
		// 'Members' Handling .. these all use safe guards, lots of them
		void	AddMember(LControl*);
		int32	CountMembers();// returns 0 if locked
		bool	RemoveMember(LControl*);// false if locked
		bool	RemoveMemberAt(int32);
		LControl*MemberAt(int32);
		
		BList*	Members();// returns NULL if locked, do not keep pointers to this data
								// I own it and WILL destroy it when I leave
								// I do not destroy the constructed LControls though.
								// I only maintain a list of pointers to them.
								// You will crash if you delete my members!
								// And be that soon as well, very soon.
		

	private:
		BList		*fMembers;
		bool		fLockedList;
		bool		fLockFunc;
		BBitmap		*fCursor;
		BPoint		fHotSpot;		


};


#endif

